@@ -1,5 +1,5 @@
 use crate::data::LineSegmentView;
-use crate::Intersects;
+use crate::{Intersects, PolygonScalar};
 
 /// Find all line segment intersections.
 ///
@@ -10,7 +10,7 @@ pub fn segment_intersections<'a, Edge, T: 'a>(
 ) -> impl Iterator<Item = (&Edge, &Edge)>
 where
   &'a Edge: Into<LineSegmentView<'a, T, 2>>,
-  T: Clone + num_traits::NumOps<T, T> + Ord + std::fmt::Debug + crate::Extended,
+  T: PolygonScalar,
 {
   pairs(edges).filter_map(|(a, b)| {
     let a_edge: LineSegmentView<'a, T, 2> = a.into();
