@@ -1,5 +1,6 @@
 mod directed_edge;
 mod intersection_set;
+mod line;
 mod line_segment;
 pub(crate) mod point;
 pub mod polygon;
@@ -8,6 +9,7 @@ mod vector;
 
 pub use directed_edge::*;
 pub use intersection_set::*;
+pub use line::*;
 pub use line_segment::*;
 pub use triangle::*;
 
