@@ -1,19 +1,26 @@
-use num_traits::*;
-
 use super::EndPoint;
 use super::ILineSegment;
 use super::LineSegment;
 use super::LineSegmentView;
 use super::Point;
-use crate::Intersects;
+use crate::{Intersects, PolygonScalar};
 
-#[derive(Clone, Copy, Eq, PartialEq, Ord, PartialOrd)]
+#[derive(Debug, Clone, Copy, Eq, PartialEq, Ord, PartialOrd)]
 // Directed edge from A to B, including A and excluding B.
 pub struct DirectedEdge<T, const N: usize> {
   pub src: Point<T, N>,
   pub dst: Point<T, N>,
 }
 
+impl<T> DirectedEdge<T, 2> {
+  pub fn contains(&self, pt: &Point<T, 2>) -> bool
+  where
+    T: Ord + PartialOrd,
+  {
+    LineSegmentView::from(self).contains(pt)
+  }
+}
+
 impl<T: Ord, const N: usize> From<DirectedEdge<T, N>> for LineSegment<T, N> {
   fn from(edge: DirectedEdge<T, N>) -> LineSegment<T, N> {
     LineSegment::new(EndPoint::Inclusive(edge.src), EndPoint::Exclusive(edge.dst))
@@ -31,7 +38,7 @@ impl<'a, T: Ord, const N: usize> From<&'a DirectedEdge<T, N>> for LineSegmentVie
 
 impl<'a, T> Intersects for &'a DirectedEdge<T, 2>
 where
-  T: Clone + Num + Ord + std::fmt::Debug + crate::Extended,
+  T: PolygonScalar,
 {
   type Result = ILineSegment<'a, T>;
   fn intersect(self, other: &'a DirectedEdge<T, 2>) -> Option<Self::Result> {
