@@ -1,11 +1,12 @@
 // use claim::debug_assert_ok;
-use num::BigInt;
-use num::BigRational;
 use num_traits::*;
+use ordered_float::OrderedFloat;
 use std::iter::Sum;
+use std::ops::Bound::*;
 use std::ops::*;
 
-use crate::data::{DirectedEdge, Point, PointLocation, TriangleView, Vector};
+use crate::data::{DirectedEdge, HalfLineSoS, Point, PointLocation, TriangleView, Vector};
+use crate::intersection::*;
 use crate::{Error, Orientation, PolygonScalar};
 
 mod iter;
@@ -201,6 +202,41 @@ impl<T> Polygon<T> {
     Ok(())
   }
 
+  pub fn locate(&self, origin: &Point<T, 2>) -> PointLocation
+  where
+    T: PolygonScalar,
+  {
+    // FIXME: Support polygons with holes.
+    if self.rings.len() != 1 {
+      panic!("FIXME: Polygon::locate should support polygons with holes.");
+    }
+    let ray = HalfLineSoS::new_directed(origin.clone(), Vector::unit_right());
+    let mut intersections = 0;
+    for edge in self.iter_boundary_edges() {
+      if edge.contains(origin) {
+        return PointLocation::OnBoundary;
+      }
+      if ray.intersect(&edge).is_some() {
+        intersections += 1;
+      }
+    }
+    if intersections % 2 == 0 {
+      PointLocation::Outside
+    } else {
+      PointLocation::Inside
+    }
+  }
+
+  pub fn triangulate(
+    &self,
+  ) -> impl Iterator<Item = (Cursor<'_, T>, Cursor<'_, T>, Cursor<'_, T>)> + '_
+  where
+    T: PolygonScalar,
+  {
+    crate::algorithms::triangulation::earclip::earclip(self)
+      .map(move |(p1, p2, p3)| (self.cursor(p1), self.cursor(p2), self.cursor(p3)))
+  }
+
   pub fn centroid(&self) -> Point<T, 2>
   where
     T: PolygonScalar,
@@ -208,12 +244,12 @@ impl<T> Polygon<T> {
     let xs: Vector<T, 2> = self
       .iter_boundary_edges()
       .map(|edge| {
-        let p = &edge.src.as_vec().cast(|v| v);
-        let q = &edge.dst.as_vec().cast(|v| v);
+        let p = edge.src.as_vec();
+        let q = edge.dst.as_vec();
         (p + q) * (p.0[0].clone() * q.0[1].clone() - q.0[0].clone() * p.0[1].clone())
       })
       .sum();
-    let three = T::from_usize(3).unwrap();
+    let three = T::from_constant(3);
     Point::from(xs / (three * self.signed_area_2x()))
   }
 
@@ -243,25 +279,6 @@ impl<T> Polygon<T> {
     (Point::new([min_x, min_y]), Point::new([max_x, max_y]))
   }
 
-  // Convert to BigRational. Center on <0,0>. Scale size such that max(width,height) = 1.
-  pub fn normalize(&self) -> Polygon<BigRational>
-  where
-    T: PolygonScalar + Into<BigInt>,
-    T::ExtendedSigned: Into<BigInt>,
-  {
-    let (min, max) = self.bounding_box();
-    let [min_x, min_y] = min.array;
-    let [max_x, max_y] = max.array;
-    let width = max_x.extend_signed() - min_x.extend_signed();
-    let height = max_y.extend_signed() - min_y.extend_signed();
-    let ratio = std::cmp::max(width, height);
-    let p = self.clone().cast(|t| BigRational::from(t.into()));
-    let centroid = p.centroid();
-    let t = Transform::translate(-Vector::from(centroid));
-    let s = Transform::uniform_scale(BigRational::new(One::one(), ratio.into()));
-    s * t * p
-  }
-
   /// Computes the area of a polygon. If the polygon winds counter-clockwise,
   /// the area will be a positive number. If the polygon winds clockwise, the area will
   /// be negative.
@@ -461,13 +478,13 @@ impl<T> Polygon<T> {
     }
   }
 
-  pub fn cast<U, F>(self, f: F) -> Polygon<U>
+  pub fn map<U, F>(self, f: F) -> Polygon<U>
   where
     T: Clone,
     U: Clone,
     F: Fn(T) -> U + Clone,
   {
-    let pts = self.points.into_iter().map(|p| p.cast(f.clone())).collect();
+    let pts = self.points.into_iter().map(|p| p.map(f.clone())).collect();
     Polygon {
       points: pts,
       ring_index: self.ring_index,
@@ -476,6 +493,26 @@ impl<T> Polygon<T> {
     }
   }
 
+  pub fn cast<U>(self) -> Polygon<U>
+  where
+    T: Clone + Into<U>,
+  {
+    let pts = self.points.into_iter().map(|p| p.cast()).collect();
+    Polygon {
+      points: pts,
+      ring_index: self.ring_index,
+      position_index: self.position_index,
+      rings: self.rings,
+    }
+  }
+
+  pub fn float(self) -> Polygon<OrderedFloat<f64>>
+  where
+    T: Clone + Into<f64>,
+  {
+    self.map(|v| OrderedFloat(v.into()))
+  }
+
   // Reverse all points between p1 and p2, inclusive of p1 and p2.
   // For example: p1, a, b, c, p2 =>
   //              p2, c, b, a, p1
@@ -555,26 +592,19 @@ impl<T> Polygon<T> {
   }
 }
 
-impl From<Polygon<BigRational>> for Polygon<f64> {
-  fn from(p: Polygon<BigRational>) -> Polygon<f64> {
-    p.cast(|r| r.to_f64().unwrap())
-  }
-}
-impl<'a> From<&'a Polygon<BigRational>> for Polygon<f64> {
-  fn from(p: &Polygon<BigRational>) -> Polygon<f64> {
-    p.clone().into()
-  }
-}
-
-impl From<Polygon<f64>> for Polygon<BigRational> {
-  fn from(p: Polygon<f64>) -> Polygon<BigRational> {
-    p.cast(|f| BigRational::from_float(f).unwrap())
-  }
-}
-
-impl<'a> From<&'a Polygon<f64>> for Polygon<BigRational> {
-  fn from(p: &Polygon<f64>) -> Polygon<BigRational> {
-    p.clone().into()
+impl Polygon<OrderedFloat<f64>> {
+  // Center on <0,0>. Scale size such that max(width,height) = 1.
+  pub fn normalize(&self) -> Polygon<OrderedFloat<f64>> {
+    let (min, max) = self.bounding_box();
+    let [min_x, min_y] = min.array;
+    let [max_x, max_y] = max.array;
+    let width = max_x - min_x;
+    let height = max_y - min_y;
+    let ratio = std::cmp::max(width, height);
+    let centroid = self.centroid();
+    let t = Transform::translate(-Vector::from(centroid));
+    let s = Transform::uniform_scale(ratio.recip());
+    s * t * self
   }
 }
 
@@ -584,6 +614,13 @@ pub struct Cursor<'a, T> {
   pub(crate) position: Position,
 }
 
+impl<'a, T> Deref for Cursor<'a, T> {
+  type Target = Point<T, 2>;
+  fn deref(&self) -> &Self::Target {
+    self.point()
+  }
+}
+
 impl<'a, T> PartialEq for Cursor<'a, T> {
   fn eq(&self, other: &Cursor<'a, T>) -> bool {
     self.position == other.position
@@ -636,7 +673,7 @@ impl<'a, T> Cursor<'a, T> {
     let trig =
       TriangleView::new_unchecked([self.prev().point(), self.point(), self.next().point()]);
     if trig.orientation() == Orientation::CounterClockWise {
-      for pt in self.next().next().to(self.prev()) {
+      for pt in self.next().next().to(Excluded(self.prev())) {
         if trig.locate(pt.point()) != PointLocation::Outside {
           return false;
         }
@@ -664,19 +701,24 @@ impl<'a, T> Cursor<'a, T> {
     self.orientation() == Orientation::CoLinear
   }
 
-  pub fn to(self: Cursor<'a, T>, end: Cursor<'a, T>) -> impl Iterator<Item = Cursor<'a, T>> + 'a {
-    CursorIter {
-      cursor_head: self,
-      cursor_tail: end.prev(),
-      exhausted: false,
-    }
-  }
-
-  pub fn to_inclusive(self, end: Cursor<'a, T>) -> CursorIter<'a, T> {
-    CursorIter {
-      cursor_head: self,
-      cursor_tail: end,
-      exhausted: false,
+  pub fn to(self: Cursor<'a, T>, end: Bound<Cursor<'a, T>>) -> CursorIter<'a, T> {
+    // FIXME: Check that self and end point to the same polygon.
+    match end {
+      Bound::Unbounded => CursorIter {
+        cursor_head: self,
+        cursor_tail: self.prev(),
+        exhausted: false,
+      },
+      Bound::Included(end) => CursorIter {
+        cursor_head: self,
+        cursor_tail: end,
+        exhausted: false,
+      },
+      Bound::Excluded(end) => CursorIter {
+        cursor_head: self,
+        cursor_tail: end.prev(),
+        exhausted: false,
+      },
     }
   }
 }
@@ -716,11 +758,6 @@ impl Position {
   }
 }
 
-// pub trait Triangulate {
-//   type Iter: Iterator<Item = (VertexId, VertexId, VertexId)>;
-//   fn triangulate(&self) -> Self::Iter;
-// }
-
 #[cfg(test)]
 pub mod tests {
   use super::*;
@@ -733,15 +770,56 @@ pub mod tests {
     prop_assert_eq!(poly.validate().err(), None);
   }
 
-  #[cfg(not(debug_assertions))] // proxy for release builds.
+  // // #[cfg(not(debug_assertions))] // proxy for release builds.
+  // #[proptest]
+  // fn normalize_props(poly: Polygon<i8>) {
+  //   // Debug builds are ~50x slower than release builds. Sigh.
+  //   let norm = poly.float().normalize();
+  //   // prop_assert_eq!(norm.centroid(), Point::zero());
+  //   let (min, max) = norm.bounding_box();
+  //   let width = max.x_coord() - min.x_coord();
+  //   let height = max.y_coord() - min.y_coord();
+  //   // prop_assert!(width == OrderedFloat(1.0) || height == OrderedFloat(1.0));
+  // }
+
+  #[test]
+  fn locate_unit_1() {
+    let poly: Polygon<i8> = Polygon::new(vec![
+      Point { array: [0, 0] },
+      Point { array: [-1, 127] },
+      Point { array: [-50, 48] },
+    ])
+    .expect("valid polygon");
+    let origin = Point { array: [79, 108] };
+
+    assert_eq!(poly.locate(&origin), PointLocation::Outside);
+  }
+
+  // Locate a point relative to a polygon. Should be identical to
+  // Polygon::locate but slower.
+  fn locate_by_triangulation<T>(poly: &Polygon<T>, origin: &Point<T, 2>) -> PointLocation
+  where
+    T: PolygonScalar,
+  {
+    for edge in poly.iter_boundary_edges() {
+      if edge.contains(origin) {
+        return PointLocation::OnBoundary;
+      }
+    }
+    for (a, b, c) in poly.triangulate() {
+      let trig = TriangleView::new([&a, &b, &c]).expect("valid triangle");
+      if trig.locate(origin) == PointLocation::Inside {
+        return PointLocation::Inside;
+      }
+    }
+    return PointLocation::Outside;
+  }
+
   #[proptest]
-  fn normalize_props(poly: Polygon<i8>) {
-    // Debug builds are ~50x slower than release builds. Sigh.
-    let norm = poly.normalize();
-    prop_assert_eq!(norm.centroid(), Point::zero());
-    let (min, max) = norm.bounding_box();
-    let width = max.x_coord() - min.x_coord();
-    let height = max.y_coord() - min.y_coord();
-    prop_assert!(width == BigRational::one() || height == BigRational::one());
+  fn locate_id_prop(poly: Polygon<i8>, origin: Point<i8, 2>) {
+    prop_assert_eq!(
+      locate_by_triangulation(&poly, &origin),
+      poly.locate(&origin)
+    )
   }
 }
