@@ -1,17 +1,13 @@
 use claim::debug_assert_ok;
-use num_bigint::BigInt;
-use num_rational::BigRational;
 use num_traits::*;
+use ordered_float::OrderedFloat;
 use rand::distributions::uniform::SampleUniform;
 use rand::distributions::{Distribution, Standard};
 use rand::seq::SliceRandom;
 use rand::Rng;
-use rand::SeedableRng;
-use std::collections::BTreeSet;
 use std::ops::*;
 
 use crate::data::{Point, PointLocation, TriangleView, Vector};
-use crate::transformation::*;
 use crate::{Error, Orientation, PolygonScalar};
 
 use super::Polygon;
@@ -84,19 +80,18 @@ where
     self.0.validate()
   }
 
+  pub fn float(self) -> PolygonConvex<OrderedFloat<f64>>
+  where
+    T: Clone + Into<f64>,
+  {
+    PolygonConvex::new_unchecked(self.0.float())
+  }
+
   /// $O(1)$
   pub fn polygon(&self) -> &Polygon<T> {
     self.into()
   }
 
-  pub fn normalize(&self) -> PolygonConvex<BigRational>
-  where
-    T: PolygonScalar + Into<BigInt>,
-    T::ExtendedSigned: Into<BigInt>,
-  {
-    PolygonConvex::new_unchecked(self.0.normalize())
-  }
-
   /// Uniformly sample a random convex polygon.
   ///
   /// The output polygon is rooted in `(0,0)`, grows upwards, and has a height and width of [`T::max_value()`](Bounded::max_value).
@@ -113,7 +108,7 @@ where
   /// # let convex: PolygonConvex<i8> = {
   /// PolygonConvex::random(3, &mut rand::thread_rng())
   /// # };
-  /// # render_polygon(&convex.normalize());
+  /// # render_polygon(&convex.float().normalize());
   /// ```
   /// <iframe src="https://web.rgeometry.org/wasm/gist/9abc54a5e2e3d33e3dd1785a71e812d2"></iframe>
   pub fn random<R>(n: usize, rng: &mut R) -> PolygonConvex<T>
@@ -149,13 +144,19 @@ where
   }
 }
 
+impl PolygonConvex<OrderedFloat<f64>> {
+  pub fn normalize(&self) -> PolygonConvex<OrderedFloat<f64>> {
+    PolygonConvex::new_unchecked(self.0.normalize())
+  }
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 // Trait Implementations
 
-impl<T: PolygonScalar> Deref for PolygonConvex<T> {
+impl<T> Deref for PolygonConvex<T> {
   type Target = Polygon<T>;
   fn deref(&self) -> &Self::Target {
-    self.polygon()
+    &self.0
   }
 }
 
@@ -186,7 +187,7 @@ where
   T: PolygonScalar + Bounded + SampleUniform + Copy,
   R: Rng + ?Sized,
 {
-  let zero: T = Zero::zero();
+  let zero: T = T::from_constant(0);
   let max: T = Bounded::max_value();
   assert!(n > 0);
   let mut pts = Vec::with_capacity(n);
@@ -239,7 +240,6 @@ where
 mod tests {
   use super::*;
 
-  use crate::testing::*;
   use proptest::prelude::*;
   use test_strategy::proptest;
 
@@ -289,6 +289,6 @@ mod tests {
   fn sum_to_zero_vector(#[strategy(2..1000)] n: i32) {
     let mut rng = rand::thread_rng();
     let vecs: Vec<Vector<i8, 2>> = random_vectors(n as usize, &mut rng);
-    prop_assert_eq!(vecs.into_iter().sum::<Vector<i8, 2>>(), Vector::zero())
+    prop_assert_eq!(vecs.into_iter().sum::<Vector<i8, 2>>(), Vector([0, 0]))
   }
 }
