@@ -1,7 +1,6 @@
 use array_init::{array_init, try_array_init};
 use num_rational::BigRational;
-use num_traits::identities::Zero;
-use num_traits::{NumOps, Signed};
+use num_traits::NumOps;
 use rand::distributions::{Distribution, Standard};
 use rand::Rng;
 use std::cmp::Ordering;
@@ -14,8 +13,7 @@ use std::ops::Neg;
 use std::ops::Sub;
 
 use crate::data::Point;
-use crate::Extended;
-use crate::Orientation;
+use crate::{Orientation, PolygonScalar};
 
 #[derive(Debug, Clone, PartialEq, Eq, PartialOrd, Ord)]
 #[repr(transparent)]
@@ -43,23 +41,26 @@ where
 
   pub fn squared_magnitude(&self) -> T
   where
-    T: Zero + AddAssign,
+    T: Sum + AddAssign,
     for<'a> &'a T: Sub<&'a T, Output = T> + Mul<&'a T, Output = T>,
   {
-    let mut total = T::zero();
-    for elt in self.0.iter() {
-      total += elt * elt;
-    }
-    total
+    self.0.iter().map(|elt| elt * elt).sum()
   }
 
-  pub fn cast<U, F>(&self, f: F) -> Vector<U, N>
+  pub fn map<U, F>(&self, f: F) -> Vector<U, N>
   where
     T: Clone,
     F: Fn(T) -> U,
   {
     Vector(array_init(|i| f(self.0[i].clone())))
   }
+
+  pub fn cast<U>(&self) -> Vector<U, N>
+  where
+    T: Clone + Into<U>,
+  {
+    Vector(array_init(|i| self.0[i].clone().into()))
+  }
 }
 
 impl<T, const N: usize> Index<usize> for Vector<T, N> {
@@ -97,11 +98,19 @@ impl<'a, const N: usize> TryFrom<Vector<f64, N>> for Vector<BigRational, N> {
 }
 
 impl<T> Vector<T, 2> {
+  // Unit vector pointing to the right.
+  pub fn unit_right() -> Vector<T, 2>
+  where
+    T: PolygonScalar,
+  {
+    Vector([T::from_constant(1), T::from_constant(0)])
+  }
+
   pub fn ccw_cmp_around(&self, p: &Vector<T, 2>, q: &Vector<T, 2>) -> Ordering
   where
-    T: Extended + Signed,
+    T: PolygonScalar,
   {
-    self.ccw_cmp_around_with(&Vector([T::one(), T::zero()]), p, q)
+    self.ccw_cmp_around_with(&Vector([T::from_constant(1), T::from_constant(0)]), p, q)
   }
   pub fn ccw_cmp_around_with(
     &self,
@@ -110,7 +119,7 @@ impl<T> Vector<T, 2> {
     q: &Vector<T, 2>,
   ) -> Ordering
   where
-    T: Extended + Signed,
+    T: PolygonScalar,
   {
     Orientation::ccw_cmp_around_with(z, &self.0, &p.0, &q.0)
   }
@@ -119,18 +128,23 @@ impl<T> Vector<T, 2> {
   // FIXME: sort by magnitude if two vectors have the same angle.
   pub fn sort_around(pts: &mut Vec<Vector<T, 2>>)
   where
-    T: Extended + Signed,
+    T: PolygonScalar,
   {
-    let origin = [T::zero(), T::zero()];
+    let origin = [T::from_constant(0), T::from_constant(0)];
     pts.sort_unstable_by(|a, b| {
-      Orientation::ccw_cmp_around_with(&Vector([T::one(), T::zero()]), &origin, &a.0, &b.0)
+      Orientation::ccw_cmp_around_with(
+        &Vector([T::from_constant(1), T::from_constant(0)]),
+        &origin,
+        &a.0,
+        &b.0,
+      )
     })
     // L.sortBy (ccwCmpAround c <> cmpByDistanceTo c)
   }
 
   pub fn cmp_along(&self, p: &Point<T, 2>, q: &Point<T, 2>) -> Ordering
   where
-    T: crate::PolygonScalar,
+    T: PolygonScalar,
   {
     // Rotate the vector 90 degrees counterclockwise.
     match Orientation::along_perp_vector(&p.array, self, &q.array) {
@@ -146,29 +160,16 @@ mod div;
 mod mul;
 mod sub;
 
-impl<T, const N: usize> Zero for Vector<T, N>
-where
-  T: NumOps + Zero + Clone,
-  // for<'c> &'c T: Add<&'c T, Output = T>,
-{
-  fn zero() -> Vector<T, N> {
-    Vector(array_init(|_| Zero::zero()))
-  }
-  fn is_zero(&self) -> bool {
-    self.0.iter().all(Zero::is_zero)
-  }
-}
-
 impl<T, const N: usize> Sum for Vector<T, N>
 where
-  T: NumOps + Zero + AddAssign + Clone,
+  T: NumOps + AddAssign + Clone + Sum,
   // for<'c> &'c T: Add<&'c T, Output = T>,
 {
   fn sum<I>(iter: I) -> Vector<T, N>
   where
     I: Iterator<Item = Vector<T, N>>,
   {
-    let mut acc = Zero::zero();
+    let mut acc = Vector(array_init(|_| std::iter::empty().sum()));
     for vec in iter {
       acc += vec;
     }
@@ -189,7 +190,6 @@ where
 #[cfg(test)]
 mod tests {
   use super::*;
-  use crate::testing::*;
 
   use proptest::prelude::*;
   use test_strategy::proptest;
