@@ -1,7 +1,7 @@
-// pub trait Intersects<T> {
-//   type Result;
-//   fn intersect(&self, other: &T) -> Option<Self::Result>;
-// }
+pub trait Intersects<T = Self> {
+  type Result;
+  fn intersect(self, other: T) -> Option<Self::Result>;
+}
 
 // // impl<T, U> Intersects<U> for T
 // // where
