@@ -112,3 +112,21 @@ impl DenseCollection {
     self.dense.iter().copied()
   }
 }
+
+///////////////////////////////////////////////////////////////////////////////
+// Iterator permutations
+
+#[cfg(test)]
+pub fn permutations<T, const N: usize>(source: [&[T]; N]) -> impl Iterator<Item = [T; N]> + '_
+where
+  T: Copy,
+{
+  let end: usize = source.iter().map(|x| x.len()).product();
+  (0..end).into_iter().map(move |mut nth| {
+    array_init::array_init(|i| {
+      let my_index = nth % source[i].len();
+      nth = nth / source[i].len();
+      source[i][my_index]
+    })
+  })
+}
